"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const mustache_1 = tslib_1.__importDefault(require("mustache"));
const path_1 = require("path");
const tar_1 = tslib_1.__importDefault(require("tar"));
const fs_1 = require("./fs");
const MUSTACHE_EXTENSION = '.mustache';
exports.CAPACITOR_VERSION = '^6.0.0';
const TEMPLATE_PATH = path_1.resolve(__dirname, '..', 'assets', 'plugin-template.tar.gz');
const WWW_TEMPLATE_PATH = path_1.resolve(__dirname, '..', 'assets', 'www-template.tar.gz');
exports.readPackageJson = async (p) => {
    const contents = await fs_1.readFile(p, { encoding: 'utf8' });
    return JSON.parse(contents);
};
exports.extractTemplate = async (dir, details, type) => {
    const templateFiles = [];
    await fs_1.mkdir(dir, { recursive: true });
    await tar_1.default.extract({
        file: type === 'PLUGIN_TEMPLATE' ? TEMPLATE_PATH : WWW_TEMPLATE_PATH,
        cwd: dir,
        filter: p => {
            if (p.endsWith(MUSTACHE_EXTENSION)) {
                templateFiles.push(p);
            }
            return true;
        },
    });
    await Promise.all(templateFiles.map(p => path_1.resolve(dir, p)).map(p => exports.applyTemplate(p, details)));
};
exports.applyTemplate = async (p, { name, 'package-id': packageId, 'class-name': className, repo, author, license, description, }) => {
    const template = await fs_1.readFile(p, { encoding: 'utf8' });
    const view = {
        CAPACITOR_VERSION: exports.CAPACITOR_VERSION,
        PACKAGE_NAME: name,
        PACKAGE_ID: packageId,
        NATIVE_NAME: packageNameToNative(name),
        CLASS: className,
        JAVA_PATH: path_1.join(packageId.split('.').join(path_1.sep), className),
        REPO_URL: repo ? repo.replace(/\/$/, '') : '',
        AUTHOR: author,
        LICENSE: license,
        DESCRIPTION: description,
    };
    const contents = mustache_1.default.render(template, view);
    const filePath = Object.entries(view).reduce((acc, [key, value]) => (value ? acc.replace(`__${key}__`, value) : acc), p.substring(0, p.length - MUSTACHE_EXTENSION.length));
    await fs_1.mkdir(path_1.dirname(filePath), { recursive: true });
    // take off the .mustache extension and write the file, then remove the template
    await fs_1.writeFile(filePath, contents, { encoding: 'utf8' });
    await fs_1.unlink(p);
};
function packageNameToNative(name) {
    name = name
        .replace(/\//g, '_')
        .replace(/-/g, '_')
        .replace(/@/g, '')
        .replace(/_\w/g, m => m[1].toUpperCase());
    return name.charAt(0).toUpperCase() + name.slice(1);
}
exports.packageNameToNative = packageNameToNative;
